<?php


/**
 * @file
 * This is a restful service class for the Atlassian SSO solution crowd.
 *
 * @author
 * John P. Warren
 */


/**
 * Provides a crowd REST service connection.
 */
class crowd_rest_client implements crowd_service {
  // Properties.
  protected $request_url;
  protected $request_headers;
  protected $request_method;
  protected $request_data;
  protected $request_service;
  protected $max_redirects;
  protected $timeout;


  /**
   * Constructor.
   *
   * Defines the base connection URL and other common properties.
   */
  function __construct() {
    // Create the full url for drupal_http_request().
    $server_address = variable_get('crowd_server', '');
    if (strpos($server_address, 'http://') !== FALSE) {
      $url_prefix = 'http://';
    }
    else {
      $url_prefix = 'https://';
    }
    if (!empty($url_prefix)) {
      $url_suffix = str_ireplace($url_prefix, '', $server_address); // ($server_data, $url);
    }
    $this->request_url = $url_prefix . rawurlencode(variable_get('crowd_application', '')) . ':' . rawurlencode(variable_get('crowd_application_password', '')) . '@' . $url_suffix . ':' . variable_get('crowd_port', '443') . variable_get('crowd_uri', '');
    $this->request_headers = array('Accept' => 'application/json', 'Content-Type' => 'application/xml');
    // Set default values for connection configuration.
    $this->max_redirects = 3;
    $this->timeout = 120.0;
  }


  /**
   * Authorize a user through the crowd server.
   *
   * @param string $username
   *   Crowd username
   * @param string $password
   *   User password
   * @return
   *   A keyed array with the Crowd token or FALSE
   */
  public function authorize($username, $password) {
    $this->request_method = 'POST';
    // Note: it would be nice to convert the XML below to a SimpleXMLElement,
    // but this is only possible with php5, and D6 still needs to support php4.
    $this->request_data = '<?xml version="1.0" encoding="UTF-8"?>
      <authentication-context>
        <username>' . htmlspecialchars($username, ENT_NOQUOTES, 'UTF-8') .'</username>
        <password>' . htmlspecialchars($password, ENT_NOQUOTES, 'UTF-8') . '</password>
        <validation-factors>
          <validation-factor>
            <name>remote_address</name>
            <value>' . ip_address() . '</value>
          </validation-factor>
        </validation-factors>
      </authentication-context>';
    //@todo add explicit json header
    $final_url = $this->request_url . '/session';
    $result = drupal_http_request($final_url, $this->request_headers, $this->request_method, $this->request_data, $this->max_redirects, $this->timeout);
    if ($result->code == '204' && $result->status_message == 'DELETE') {
      return TRUE;
    }
    if ($result->code == '200' || $result->code == '201') {
      $data = json_decode($result->data);
      $return_data = array('token' => $data->token);
      return $return_data;
    }
    else {
      return FALSE;
    }
  }


  /**
   * Log the user out from the crowd server.
   *
   * @param string $user_token
   *   The active crowd token for a user.
   * @return
   *   Boolen TRUE on success for FALSE on failure.
   */
  public function logout($user_token) {
    global $user;
    $this->request_method = 'DELETE';
    $final_url = $this->request_url . '/session/' . rawurlencode($user_token);
    $this->request_data = '';
    // Invalidate the token for this user for all application clients in Crowd.
    $result = drupal_http_request($final_url, $this->request_headers, $this->request_method, $this->request_data, $this->max_redirects, $this->timeout);
    if ($result->code == '204' && $result->status_message == 'No Content') {
      return TRUE;
    }
    else {
      watchdog('crowd', 'REST: Logout of crowd failed, messages: @data', array('@data' => serialize($result)), WATCHDOG_ERROR);
      return FALSE;
    }
  }


  /**
   * Get user data based on an active session token.
   *
   * @param string $user_token
   *   The active crowd token for a user.
   * @return
   *   An array containing the user data points or FALSE on failure.
   */
  public function user_get($user_token) {
    $this->request_method = 'GET';
    $final_url = $this->request_url . '/session/' . rawurlencode($user_token) . '?' . drupal_query_string_encode(array('expand' => 'user'));
    $result = drupal_http_request($final_url, $this->request_headers, $this->request_method, '', $this->max_redirects, $this->timeout);
    if ($result->code == '200' || $result->code =='201') {
      $crowd_user = json_decode($result->data);
      $crowd_data = array();
      $crowd_data['username'] = $crowd_user->user->name;
      $crowd_data['email'] = $crowd_user->user->email;
      $crowd_data['active'] = $crowd_user->user->active;
      $crowd_data['first-name'] = $crowd_user->user->{'first-name'};
      $crowd_data['last-name'] = $crowd_user->user->{'last-name'};
      $crowd_data['display-name'] = $crowd_user->user->{'display-name'};
      $crowd_data['attributes'] = $crowd_user->user->attributes;
      return $crowd_data;
    }
    else {
      watchdog('crowd', 'REST: Crowd authentication failed to get the user object from token.', array(), WATCHDOG_ERROR);
      return FALSE;
    }
  }


  /**
   * Check to see if the user is already logged into Crowd.
   *
   * @param string $user_token
   *   The active crowd token for a user.
   * @return
   *   Boolean TRUE if the user is logged-in to crowd or FALSE if not.
   */
  public function crowd_is_logged_in($user_token) {
    $this->request_method = 'GET';
    $final_url = $this->request_url . '/session/' . rawurlencode($user_token);
    $result = drupal_http_request($final_url, $this->request_headers, $this->request_method, '', $this->max_redirects, $this->timeout);
    $this->request_method = 'POST';
    $data = json_decode($result->data);
    if ($data->user->active ) { // Should always be true or false
      return TRUE;
    }
    else {
      return FALSE;
    }
  }


  /**
   * Get all active groups from the crowd server.
   *
   * @return
   *   A serial array of the groupnames or FALSE on failure.
   */
  public function list_groups() {
    $this->request_method = 'POST';
    // Note: it would be nice to convert the XML below to a SimpleXMLElement,
    // but this is only possible with php5, and D6 still needs to support php4.
    $group_list_xml = '<?xml version="1.0" encoding="UTF-8"?>
    <property-search-restriction>
      <property>
        <name>active</name>
        <type>BOOLEAN</type>
      </property>
      <match-mode>EXACTLY_MATCHES</match-mode>
      <value>true</value>
    </property-search-restriction>';
    $this->request_data = $group_list_xml;
    $final_url = $this->request_url . '/search?' . drupal_query_string_encode(array('entity-type' => 'group'));
    $result = drupal_http_request($final_url, $this->request_headers, $this->request_method, $this->request_data, $this->max_redirects, $this->timeout);
    if ($result->code == '200' || $result->code =='201') {
      $data = json_decode($result->data);
      $results = array();
      foreach ($data->groups as $key => $value) {
        $results[] = $value->name;
      }
      return $results;
    }
    else {
      watchdog('crowd', 'REST: Crowd group listing failed. Error: %data', array('%data' => serialize($result)), WATCHDOG_ERROR);
      return FALSE;
    }
  }


  /**
   * Get the crowd groups for a specific user.
   *
   * @param string $name
   *   The crowd username.
   * @return
   *   An array containing all of the given user's groups or FALSE on failure.
   */
  public function list_user_groups($name) {
    $this->request_method = 'GET';
    $this->request_data .= '';
    $final_url = $this->request_url . '/user/group/nested?' . drupal_query_string_encode(array('username' => $name));
    $result = drupal_http_request($final_url, $this->request_headers, $this->request_method, $this->request_data, $this->max_redirects, $this->timeout);
    $this->request_method = 'POST';
    if ($result->code == '200' || $result->code =='201') {
      $user_crowd_data = json_decode($result->data);
      $user_crowd_groups = array();
      foreach ($user_crowd_data->groups as $key => $value) {
        $user_crowd_groups[] = $value->name;
      }
      return $user_crowd_groups;
    }
    else {
      watchdog('crowd', 'REST: Crowd user group listing failed. Error: %data', array('%data' => serialize($result)), WATCHDOG_ERROR);
      return FALSE;
    }
  }
}
